from PySZG import *
import random
import time

#I'm lazy
vr = arVector3

class navigator:
    def __init__(self):
	self.node = None
	self.lastTime = -1
	self.speed = 3.0
	self.threshold = 0.4

    def update(self, framework, time):
	if self.lastTime < 0:
	    self.lastTime = time
	    return
	
	wandDir = ar_ERM(framework.getMatrix(1))*arVector3(0,0,-1)
	wandTransDir = ar_ERM(framework.getMatrix(1))*arVector3(1,0,0)

	deltaX = framework.getAxis(0)
	deltaY = framework.getAxis(1)
	
	if abs(deltaX) < self.threshold:
	    deltaX = 0
	else:
	    deltaX = (deltaX - self.threshold) / (1.0 - self.threshold)
	if abs(deltaY) < self.threshold:
	    deltaY = 0
	else:
	    deltaY = (deltaY - self.threshold) / (1.0 - self.threshold)
	t = (wandDir * deltaY + wandTransDir * deltaX) * (time - self.lastTime) * self.speed 
	self.node.set(ar_TM(-t)*self.node.get())
	self.lastTime = time

#Used to read in input
class tool:
    def __init__(self):
        #These are set true when a button is pressed
        #Must be reset outside of this class
	self.pressLeft = False
        self.pressRight = False
        self.pressLeft2 = False
        self.pressRight2 = False

    #Called by eventprocess
    def update(self, framework, event):
        #Only respond to button events when it is a
        #Mousedown event
	if event.getType() == AR_EVENT_BUTTON and event.getButton() == 0:
	    '''
	    getIndex = which button
            In input simulator, different button sets
            are selected by hitting space bar while in
            mode 4 or 5.
            '''
	    if event.getIndex() == 0:
		self.pressLeft = True
	    elif event.getIndex() == 1:
		self.pressRight = True
            elif event.getIndex() == 2:
		self.pressLeft2 = True
	    elif event.getIndex() == 3:
		self.pressRight2 = True

    #SZG Framework calls this with an eventqueue
    def eventProcessing(self, framework, eventQueue):
        while not eventQueue.empty():
            e = eventQueue.popNextEvent()
            self.update(framework, e)
        return 1

def drawPlane(parentNode):
    ptran = parentNode.new("transform")
    ptran.set(ar_SM(.3))
    #Left wing
    points = ptran.new("points")
    points.set([vr(-4,0,4),vr(0,0,-4),vr(-2,.001,0),vr(-.1,0,4)])
    normals = points.new("normal3")
    normals.set([vr(0,1,0),vr(0,1,0),vr(0,1,0),vr(0,1,0),vr(0,1,0)])
    indx = normals.new("index")
    indx.set([0,3,2,1,0]) 
    triangle1 = indx.new("drawable")
    triangle1.setDrawable(DG_TRIANGLE_STRIP,3)
    #Right wing
    points = ptran.new("points")
    points.set([vr(4,0,4),vr(0,0,-4),vr(2,.001,0),vr(.1,0,4)])
    normals = points.new("normal3")
    normals.set([vr(0,1,0),vr(0,1,0),vr(0,1,0),vr(0,1,0),vr(0,1,0)])
    indx = normals.new("index")
    indx.set([0,1,2,3,0])
    triangle1 = indx.new("drawable")
    triangle1.setDrawable(DG_TRIANGLE_STRIP,3)
    #Left bottom 
    points = ptran.new("points")
    points.set([vr(0,0,-4),vr(-.1,0,4),vr(-.026,-1,2),vr(0,-2,4)])
    normals = points.new("normal3")
    test = vr(-16,-4,-3).normalize()
    normals.set([test,test,test,test,test])
    indx = normals.new("index")
    indx.set([0,1,2,3,0])
    triangle1 = indx.new("drawable")
    triangle1.setDrawable(DG_TRIANGLE_STRIP,3)
    #Right bottom 
    points = ptran.new("points")
    points.set([vr(0,0,-4),vr(.1,0,4),vr(.026,-1,2),vr(0,-2,4)])
    normals = points.new("normal3")
    test = vr(16,-4,-3).normalize()
    normals.set([test,test,test,test,test])
    indx = normals.new("index")
    indx.set([0,3,2,1,0])
    triangle1 = indx.new("drawable")
    triangle1.setDrawable(DG_TRIANGLE_STRIP,3)

f = arDistSceneGraphFramework();

#Make the tool so we can read input
theTool = tool()

#Set the event queue callback
f.setEventQueueCallback(theTool.eventProcessing)

if len(sys.argv) < 3:
    print("Incorrect argument format:/nworldpart.py partname sphere|hexahedron|plane")
    sys.exit()

if f.init(sys.argv) != 1:
    #Oh no....framework failed to initialize
    sys.exit()

#Must be operating a phleet
if f.getStandalone():
    print("This program only works in cluster mode. Please log in.")
    sys.exit()

#This is where the fun stuff begins
peer = arGraphicsPeer()
#Set this peer's name
peer.setName(sys.argv[1])
print("Using name: " + sys.argv[1])
#Don't queue the data
peer.queueData(0)
peer.init(f.getSZGClient())
peer.setTexturePath(f.getSZGClient().getAttribute("SZG_RENDER","texture_path"))
peer.loadAlphabet(f.getSZGClient().getAttribute("SZG_RENDER","text_path"))
#Set the peer to use our database
peer.setBridge(f.getDatabase())

nav = navigator()
nav.node = f.getDatabase().getRoot().new("transform")

#Set the root node from which we share
peer.setBridgeRootMapNode(nav.node)

if peer.start() != 1:
	sys.exit()
if f.start() != 1:
	sys.exit()

'''
Lighting and the floor are attached to the nav node
not the worldRoot.  Therefore, they are not shared.
'''
# NOTE!!!  When using shared worlds, shared nodes MUST
# be added below peer.getRoot() NOT  the node that was
# set in peer.setBridgeRootMapNode !!!!

l = nav.node.new("light")
l2 = nav.node.new("light")

light = arLight()
light.position = arVector4(-15,15,0,1)
light.diffuse = arVector3(1,1,1)
light.ambient = arVector3(0,0,0)
light.lightID = 0
l.set(light)

light = arLight()
light.position = arVector4(20,20,0,0)
light.ambient = arVector3(.1,.1,.1)
light.lightID = 1
l2.set(light)

#Create our part of the world
#For some reason, unless we name this a unique name
#The transform node gets replaced by the next parts
#transform in buildworld.py
worldRoot = peer.getRoot().new("transform", sys.argv[1])

mat = worldRoot.new("material")
material = arMaterial()
material.specular = arVector3(0,0,0)
material.diffuse = arVector3(1,0,0)
material.ambient = arVector3(1,0,0)
mat.set(material)
tran = mat.new("transform")
tran.set(nav.node.get().inverse())
tran2 = tran.new("transform")
tran2.set(ar_TM(0,5,0))

if sys.argv[2] == "sphere":
    part = arSphereMesh(50)
    part.attachMesh(tran2)
elif sys.argv[2] == "hexahedron":
    part = arCubeMesh()
    part.attachMesh(tran2)
else:
    drawPlane(tran2)

while 1:
    f.setViewer()
    f.setPlayer()
    f.processEventQueue()

    nav.update(f, time.clock())
    tran.set(nav.node.get().inverse())
        
    if theTool.pressLeft:
        theTool.pressLeft = False
        color = arVector3(random.random(), random.random(), random.random())
        material.diffuse = color
        material.ambient = color
        mat.set(material)
    
    #time.sleep(0.01)
    f.loadNavMatrix()
